
package odbdatebugtestcase;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.TimeZone;
import java.util.UUID;
import javax.jdo.annotations.Index;
import javax.persistence.Entity;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Id;
import javax.persistence.Persistence;
import javax.persistence.Query;
import javax.persistence.Temporal;
import javax.persistence.TypedQuery;

public class OdbDateBugTestCase {

    public static void main(String[] args) {
        
        // time zone with problems
        TimeZone.setDefault(TimeZone.getTimeZone("Europe/Moscow"));
//        TimeZone.setDefault(TimeZone.getTimeZone("GMT+04:00"));
    
        // init connection
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory("$objectdb/db/persons.odb");
        EntityManager entityManager = entityManagerFactory.createEntityManager();
        entityManager.getMetamodel().entity(Person.class);
        
        // generate persons
        Query countQuery = entityManager.createQuery("select count(p.id) from Person p");
        long personsCount = (long) countQuery.getSingleResult();
        
        if (personsCount == 0) {
            entityManager.getTransaction().begin();
            for (int index = 0; index < 10000; index++) {
                Person person = new Person();
                person.setBirthday(new Date());
                entityManager.persist(person);
            }
            entityManager.getTransaction().commit();
        }
        
        // create dates for update
        Calendar calendar = Calendar.getInstance();
        System.out.println(calendar.getTimeZone().toString());
        List<Date> dates = new ArrayList<Date>();
        calendar.set(1980, 2, 1);
        dates.add(calendar.getTime());
        calendar.set(1981, 2, 31);
        dates.add(calendar.getTime());
        calendar.set(1981, 3, 1);
        dates.add(calendar.getTime());

        // update birthday field
        for (Date newBirthday : dates) {
            System.out.println("Set new birthday:" + newBirthday.toString());
            try {
                entityManager.getTransaction().begin();
                TypedQuery<Person> personQuery = entityManager.createQuery("select p from Person p", Person.class);
                List<Person> persons = personQuery.getResultList();
                for(Person person : persons) {
                    person.setBirthday(newBirthday);
                }
                entityManager.getTransaction().commit();
            } catch (Exception ex) {
                ex.printStackTrace();
            }
        }
        
        entityManager.close();
        entityManagerFactory.close();
        
    }
    
    @Entity
    public static class Person {

        private static final long serialVersionUID = 1L;

        // -------------------- persistent fields ----------------------------------

        @Id
        private String id = UUID.randomUUID().toString();

        @Index
        @Temporal(javax.persistence.TemporalType.DATE)
        private Date birthday;

        // -------------------- accessors ------------------------------------------

        public String getId() {
            return id;
        }

        public Date getBirthday() {
            return birthday;
        }

        public void setBirthday(Date birthday) {
            this.birthday = birthday;
        }

        // -------------------- override -------------------------------------------
        @Override
        public String toString() {
            return "Person : { id:" + id + ", birthday:" + birthday.toString() + " }";
        }

    }
    
}
